(document:surround "/std/frame")
(document:envelop with-translation _ "alterator-pkg")


;;; Features

(define *pkg-lists* (make-cell '()))
(define *pkg-conflicts* (make-cell '()))
(define *pkg-licenses* (make-cell '()))

;;; Features: Lists

(define (pkg-install-list)
  (string-join
   (fold (lambda (x1 x2 result)
           (if (cdr x2) (cons x1 result) result))
         '()
         (cell-ref *pkg-lists*)
         (packages checklist-rows))
   " "))

(define (label+required x)
  (cons (woo-get-option x 'label) (woo-get-option x 'required)))

(define (make-pkg-lists pkg-groups)
  (cell-set! *pkg-lists* (map (lambda(x) (woo-get-option x 'lists)) pkg-groups)))

(define (make-pkg-licenses pkg-groups)
  (cell-set! *pkg-licenses* (map (lambda(x) (woo-get-option x 'license)) pkg-groups)))

;;; Features: Conflicts

(define (name+conflicts x)
  (cons (woo-get-option x 'name)
        (woo-get-option x 'conflicts)))

(define (get-index lst)
  (lambda(x)
    (list-index (lambda (y) (string=? x y)) lst)))

(define (make-pkg-conflicts pkg-groups)
  (let* ((lst (map name+conflicts pkg-groups))
         (names (map car lst))
         (conflicts (map cdr lst)))
    (cell-set! *pkg-conflicts* (map (get-index names) conflicts))))

(define (pkg-license-name num)
  (list-ref (cell-ref *pkg-licenses*) num))

(define (check-pkg-license num status)
  (let ((name (pkg-license-name num)))
    (or (not status)
        (string-null? name)
	(or (document:popup "/pkg/license" 'name name)
	    (begin (packages checklist-row-item num #f)
	           #f)))))

(define (pkg-conflicts-num num)
  (list-ref (cell-ref *pkg-conflicts*) num))

(define (pkg-conflicts-name num)
  (car (list-ref (packages checklist-rows) num)))

(define (check-pkg-conflicts num status)
  (let* ((c-num (pkg-conflicts-num num)))
    (and status
         (number? c-num)
         (packages checklist-row-item c-num)
         (begin (document:popup-critical
                 (format #f (_ "Has conflict with ~S") (pkg-conflicts-name c-num)))
                (packages checklist-row-item num #f)
		#t))))

;;; Features: Disk usage

(define (string-prefix-at-pos str prefix pos)
  (and (string-prefix? prefix str)
       (list-ref (string-cut str #\:) pos)))

(define (pkg-size-label msg)
  (size text (format #f "<b>~A</b>:~A" (_ "Required disk space") msg)))

(define (pkg-message-handler msg)
  (and-let* ((msg (cond-plistq 'message msg #f)))
            (cond
             ((string=? msg "pkg-size:start")
              (pkg-size-label (_ "calculating...")))
             ((string-prefix-at-pos msg "apt-get:status:disk-size:" 3) =>
              pkg-size-label)
             ((string-prefix-at-pos msg "pkg-size:finish" 2) =>
              (lambda(result)
                (and (string=? result "100") (pkg-size-label (_ "unable to calculate"))))))))


(define (pkg-error-handler msg)
  (and-let* ((error-text (cond-plistq 'error msg #f)))
       (format #t "error-text:~S~%" error-text)))

(define (make-pkg-size)
  (add-mailbox pkg-message-handler)
  (add-mailbox pkg-error-handler)
  (pkg-size-label "0Kb"))

(define (remove-pkg-size)
  (remove-mailbox pkg-message-handler)
  (remove-mailbox pkg-error-handler)
  (woo-write "/pkg-size" 'lists ""))

(define (check-pkg-size)
  (woo-write "/pkg-size" 'lists (pkg-install-list)))

;;; UI

(gridbox columns "10;80;10"
         ;;
         (spacer)
         (groupbox (_ "Select packages you want to install")
		   (document:id packages (checklist
					   (when clicked
					     (let* ((num (packages current))
						    (status (packages checklist-row-item num)))
					       (and (check-pkg-license num status)
						    (not (check-pkg-conflicts num status))
						    (check-pkg-size)))))))
         (spacer)
         ;;
         (spacer)
         (document:id size (label (_ "Preparing...")))
         (spacer))
                   

;;; Logic

(frame:on-next (thunk (remove-pkg-size)
                      (or (null? (cell-ref *pkg-lists*))
		          (begin (frame:replace "/pkg/install" 'lists (pkg-install-list))
		                 'cancel))))

(document:root
 (when loaded
   (woo-catch/message
    (thunk
     (let* ((pkg-groups (woo-list "/pkg-groups")))
       (if (null? pkg-groups)
           (frame:skip)
           (begin 
                  (splash-message (_ "Please wait..."))
                  (document:release)

	          ;;update cache
                  (woo-write "/pkg-update")
                  (woo-write "/autoinstall/pkg-update")

	          (make-pkg-lists pkg-groups)
	          (make-pkg-conflicts pkg-groups)
	          (make-pkg-licenses pkg-groups)
		  (make-pkg-size)
                  (packages checklist-rows (map label+required pkg-groups))
		  (check-pkg-size)

		  (splash-message ""))))))))
